import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import '../features/call/call_page.dart';
import '../features/call/incoming_call_page.dart';
import '../services/contact_service.dart';

class CallService {
  static const MethodChannel _channel = MethodChannel('call_service');
  static String? currentPhoneNumber;
  static String? currentDisplayName;
  static Uint8List? currentThumbnail;
  static bool _isCallPageVisible = false;
  static String? _currentCallState;
  static Map<String, dynamic>? _pendingCall;
  static bool wasPhoneLocked = false;
  final ContactService _contactService = ContactService();

  static final GlobalKey<NavigatorState> navigatorKey = GlobalKey<NavigatorState>();

  CallService() {
    _channel.setMethodCallHandler((call) async {
      print('CallService: Received method ${call.method} with args ${call.arguments}');
      switch (call.method) {
        case "callAdded":
          final phoneNumber = call.arguments["callId"] as String;
          final state = call.arguments["state"] as String;
          currentPhoneNumber = phoneNumber.replaceFirst('tel:', '');
          await _fetchContactInfo(currentPhoneNumber!);
          print('CallService: Call added, number: $currentPhoneNumber, state: $state');
          _handleCallState(state);
          break;
        case "callStateChanged":
          final state = call.arguments["state"] as String;
          wasPhoneLocked = call.arguments["wasPhoneLocked"] as bool? ?? false;
          print('CallService: State changed to $state, wasPhoneLocked: $wasPhoneLocked');
          _handleCallState(state);
          break;
        case "callEnded":
        case "callRemoved":
          wasPhoneLocked = call.arguments["wasPhoneLocked"] as bool? ?? false;
          print('CallService: Call ended/removed, wasPhoneLocked: $wasPhoneLocked');
          _closeCallPage();
          if (wasPhoneLocked) {
            _channel.invokeMethod("callEndedFromFlutter");
          }
          currentPhoneNumber = null;
          currentDisplayName = null;
          currentThumbnail = null;
          _currentCallState = null;
          break;
        case "incomingCallFromNotification":
          final phoneNumber = call.arguments["phoneNumber"] as String;
          wasPhoneLocked = call.arguments["wasPhoneLocked"] as bool? ?? false;
          currentPhoneNumber = phoneNumber;
          await _fetchContactInfo(currentPhoneNumber!);
          print('CallService: Incoming call from notification: $phoneNumber, wasPhoneLocked: $wasPhoneLocked');
          _handleIncomingCall(phoneNumber);
          break;
      }
    });
  }

  Future<void> _fetchContactInfo(String phoneNumber) async {
    try {
      final contacts = await _contactService.fetchContacts();
      final normalizedPhoneNumber = _normalizePhoneNumber(phoneNumber);
      for (var contact in contacts) {
        for (var phone in contact.phones) {
          if (_normalizePhoneNumber(phone.number) == normalizedPhoneNumber) {
            currentDisplayName = contact.displayName;
            currentThumbnail = contact.thumbnail;
            return;
          }
        }
      }
      currentDisplayName = phoneNumber;
      currentThumbnail = null;
    } catch (e) {
      print('CallService: Error fetching contact info: $e');
      currentDisplayName = phoneNumber;
      currentThumbnail = null;
    }
  }

  String _normalizePhoneNumber(String number) {
    return number.replaceAll(RegExp(r'[\s\-\(\)]'), '');
  }

  void _handleIncomingCall(String phoneNumber) {
    final context = navigatorKey.currentContext;
    if (context == null) {
      print('CallService: Context is null, queuing incoming call: $phoneNumber');
      _pendingCall = {"phoneNumber": phoneNumber};
      Future.delayed(Duration(milliseconds: 500), () => _checkPendingCall());
    } else {
      _navigateToIncomingCallPage(context);
    }
  }

  void _checkPendingCall() {
    if (_pendingCall != null) {
      final context = navigatorKey.currentContext;
      if (context != null) {
        print('CallService: Processing queued call: ${_pendingCall!["phoneNumber"]}');
        currentPhoneNumber = _pendingCall!["phoneNumber"];
        _navigateToIncomingCallPage(context);
        _pendingCall = null;
      } else {
        print('CallService: Context still null, retrying...');
        Future.delayed(Duration(milliseconds: 500), () => _checkPendingCall());
      }
    }
  }

  void _handleCallState(String state) {
    final context = navigatorKey.currentContext;
    if (context == null) {
      print('CallService: Navigator context is null, cannot navigate');
      return;
    }
    if (_currentCallState == state) {
      print('CallService: State $state already handled, skipping');
      return;
    }
    _currentCallState = state;

    if (state == "disconnected" || state == "disconnecting") {
      _closeCallPage();
    } else if (state == "active" || state == "dialing") {
      _navigateToCallPage(context);
    } else if (state == "ringing") {
      _navigateToIncomingCallPage(context);
    }
  }

  void _navigateToCallPage(BuildContext context) {
    final currentRoute = ModalRoute.of(context)?.settings.name;
    print('CallService: Navigating to CallPage. Visible: $_isCallPageVisible, Current Route: $currentRoute');
    if (_isCallPageVisible && currentRoute == '/call') {
      print('CallService: CallPage already visible, skipping navigation');
      return;
    }
    if (_isCallPageVisible && currentRoute == '/incoming_call') {
      print('CallService: Replacing IncomingCallPage with CallPage');
      Navigator.pop(context);
    }
    Navigator.pushReplacement(
      context,
      MaterialPageRoute(
        settings: const RouteSettings(name: '/call'),
        builder: (context) => CallPage(
          displayName: currentDisplayName ?? currentPhoneNumber!,
          phoneNumber: currentPhoneNumber!,
          thumbnail: currentThumbnail,
        ),
      ),
    ).then((_) {
      print('CallService: CallPage popped');
      _isCallPageVisible = false;
    });
    _isCallPageVisible = true;
  }

  void _navigateToIncomingCallPage(BuildContext context) {
    final currentRoute = ModalRoute.of(context)?.settings.name;
    print('CallService: Navigating to IncomingCallPage. Visible: $_isCallPageVisible, Current Route: $currentRoute');
    if (_isCallPageVisible && currentRoute == '/incoming_call') {
      print('CallService: IncomingCallPage already visible, skipping navigation');
      return;
    }
    if (_isCallPageVisible && currentRoute == '/call') {
      print('CallService: CallPage visible, not showing IncomingCallPage');
      return;
    }
    Navigator.push(
      context,
      MaterialPageRoute(
        settings: const RouteSettings(name: '/incoming_call'),
        builder: (context) => IncomingCallPage(
          displayName: currentDisplayName ?? currentPhoneNumber!,
          phoneNumber: currentPhoneNumber!,
          thumbnail: currentThumbnail,
        ),
      ),
    ).then((_) {
      print('CallService: IncomingCallPage popped');
      _isCallPageVisible = false;
    });
    _isCallPageVisible = true;
  }

  void _closeCallPage() {
    final context = navigatorKey.currentContext;
    if (context == null) {
      print('CallService: Cannot close page, context is null');
      return;
    }
    print('CallService: Attempting to close call page. Visible: $_isCallPageVisible');
    if (!_isCallPageVisible) {
      print('CallService: CallPage not visible, skipping pop');
      return;
    }
    if (Navigator.canPop(context)) {
      print('CallService: Popping CallPage. Current Route: ${ModalRoute.of(context)?.settings.name}');
      Navigator.pop(context);
      _isCallPageVisible = false;
    } else {
      print('CallService: Cannot pop, no routes to pop');
    }
  }

  Future<void> makeGsmCall(
    BuildContext context, {
    required String phoneNumber,
    String? displayName,
    Uint8List? thumbnail,
  }) async {
    try {
      currentPhoneNumber = phoneNumber;
      currentDisplayName = displayName ?? phoneNumber;
      currentThumbnail = thumbnail;
      if (displayName == null || thumbnail == null) {
        await _fetchContactInfo(phoneNumber);
      }
      print('CallService: Making GSM call to $phoneNumber');
      final result = await _channel.invokeMethod('makeGsmCall', {"phoneNumber": phoneNumber});
      print('CallService: makeGsmCall result: $result');
      if (result["status"] != "calling") {
        ScaffoldMessenger.of(context).showSnackBar(
          SnackBar(content: Text("Failed to initiate call")),
        );
        return;
      }
      _handleCallState("dialing");
    } catch (e) {
      print("CallService: Error making call: $e");
      ScaffoldMessenger.of(context).showSnackBar(
        SnackBar(content: Text("Error making call: $e")),
      );
      rethrow;
    }
  }

  Future<void> hangUpCall(BuildContext context) async {
    try {
      print('CallService: Hanging up call');
      final result = await _channel.invokeMethod('hangUpCall');
      print('CallService: hangUpCall result: $result');
      if (result["status"] != "ended") {
        ScaffoldMessenger.of(context).showSnackBar(
          SnackBar(content: Text("Failed to end call")),
        );
      } else {
        _closeCallPage();
      }
    } catch (e) {
      print("CallService: Error hanging up call: $e");
      ScaffoldMessenger.of(context).showSnackBar(
        SnackBar(content: Text("Error hanging up call: $e")),
      );
      rethrow;
    }
  }
}